/**
 * vim: set ts=4 :
 * =============================================================================
 * SourceMod (C)2004-2008 AlliedModders LLC.  All rights reserved.
 * =============================================================================
 *
 * This file is part of the SourceMod/SourcePawn SDK.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, version 3.0, as published by the
 * Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * As a special exception, AlliedModders LLC gives you permission to link the
 * code of this program (as well as its derivative works) to "Half-Life 2," the
 * "Source Engine," the "SourcePawn JIT," and any Game MODs that run on software
 * by the Valve Corporation.  You must obey the GNU General Public License in
 * all respects for all other code used.  Additionally, AlliedModders LLC grants
 * this exception to all derivative works.  AlliedModders LLC defines further
 * exceptions, found in LICENSE.txt (as of this writing, version JULY-31-2007),
 * or <http://www.sourcemod.net/license.php>.
 *
 */

#if defined _weaponmodel_included
  #endinput
#endif
#define _weaponmodel_included

/********************
BEFORE YOU HOOK OR SET, NO NEED TO CHECK THE WEAPON OR CLIENT IS HOOKED OR NOT, ALREADY DID IN EXTENSION.
*********************/

/*
 * Hook a weapon model.
 *
 * @param entity		        Weapon index.
 * @param model				New wepaon modle path.
 * @noreturn
 * @error				Invalid weapon index.
 */
native WeaponModel_Hook(entity, const String:model[]);

/*
 * UnHook a weapon model.
 *
 * @param entity		        Weapon index.
 * @noreturn
 * @error				Invalid weapon index.
 */
native WeaponModel_UnHook(entity);

/*
 * Return the weapon is hooked or not.
 *
 * @param entity		        Weapon index.
 * @return				true/false.
 * @error				Invalid weapon index.
 */
native bool:WeaponModel_IsHooked(entity);

/*
 * Change Hooked wepaon model.
 *
 * @param entity		        Weapon index.
 * @param model				wepaon modle path.
 * @noreturn
 * @error				Invalid weapon index.
 */
native WeaponModel_SetModel(entity, const String:model[]);

/*
 * Hook player speed. 
 * important notes: 
 * in css, the default max speed is 320. you can change it by using sv_maxspeed 1000.
 * but the max speed for alive player is 400!!
 *
 * @param client		        client index.
 * @param speed				new speed.
 * @noreturn
 * @error				Invalid client index.
 */
native WeaponModel_HookPlayerSpeed(client, Float:speed);

/*
 * UnHook player speed. Auto unhook when client disconnect.
 *
 * @param client		        client index.
 * @noreturn
 * @error				Invalid client index.
 */
native WeaponModel_UnHookPlayerSpeed(client);

/*
 * Set hooked player speed.
 *
 * @param client		        client index.
 * @param speed				new speed.
 * @noreturn
 * @error				Invalid client index.
 */
native WeaponModel_SetPlayerSpeed(client, Float:speed);

/*
 * Return the client is hooked or not.
 *
 * @param client		        client index.
 * @return				true/false.
 * @error				Invalid client index.
 */
native bool:WeaponModel_IsPlayerHooked(client);

/**
 * Do not edit below this line!
 */
public Extension:__ext_weaponmodel = 
{
	name = "WeaponModel",
	file = "weaponmodel.ext",
#if defined AUTOLOAD_EXTENSIONS
	autoload = 1,
#else
	autoload = 0,
#endif
#if defined REQUIRE_EXTENSIONS
	required = 1,
#else
	required = 0,
#endif
};
